/* 
 * ghost.c
 *
 * OpenGL 3D model of pacman ghost
 *
 * The ghost is 2.1 * RADIUS high
 *              2 * RADIUS wide
 */

#include "SDL_opengl.h"

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "SDL.h"

#define SPRITE
#include "video.h"
#include "video3d.h"

#define RADIUS (1.0)

#define LIST(x) (displaylists + (x))

enum {X, Y, Z} axes;
enum {HEAD, SKIRT, MAXLIST} lists;

static GLuint displaylists;

static float head[SLICES][STACKS][3];
static float skirt[SLICES][3];



int construct_ghost (void) {
    int i,j;
    double alpha, beta;
    
    displaylists = glGenLists(MAXLIST);
    
    /* head */
    /* calulate points for hemisphere head */
    for (i=0;i<SLICES;i++) {
        for(j=0;j<STACKS;j++) {
	    alpha = j * M_PI / STACKS / 2.0;
	    beta  = i * 2.0 * M_PI / SLICES;
	    
	    head[i][j][X] = RADIUS * cos(alpha) * cos(beta);
	    head[i][j][Y] = RADIUS * cos(alpha) * sin(beta);
	    head[i][j][Z] = RADIUS * sin(alpha);
        }
    }
    
    /* draw head here */
    glNewList(LIST(HEAD), GL_COMPILE);
    
    /* main bit of hemisphere */
    for (j=0; j<STACKS-1; j++) {
        glBegin(GL_QUAD_STRIP);
        for (i=0; i<SLICES-1; i++) {
            glVertex3fv(head[i][j]);
            glVertex3fv(head[i][j+1]);
        }
        glVertex3fv(head[0][j]);
        glVertex3fv(head[0][j+1]);
        glEnd();
    }

    /* top of hemisphere */
    glBegin(GL_TRIANGLE_FAN);
    glVertex3f(0,RADIUS,0);
    for(i=SLICES-1; i>=0; i--) {
        glVertex3fv(head[i][STACKS-1]);
    }
    glVertex3fv(head[SLICES-1][STACKS-1]);
    glEnd();
    glEndList();
    
    /* skirt */
    /* calculate points for skirt */
    /* there are eight full waves on the skirt */
    for (i=0;i<SLICES;i++) {
        alpha = (i * 8 * 2 * M_PI / SLICES) - (M_PI / 2.0);

        skirt[i][X] = head[i][0][X];
	skirt[i][Y] = head[i][0][Y];
        skirt[i][Z] = head[i][0][Z] - (1.0 * RADIUS) + (0.1 * RADIUS * sin(alpha));

    }
    
    /* draw skirt here */
    /* the skirt is a quad strip stretching from the list of points in the
     * skirt array to the bottom row of the head */

    glNewList(LIST(SKIRT), GL_COMPILE);
    glBegin(GL_QUAD_STRIP);
    
    for (i=0;i<SLICES;i++) {
        glVertex3fv(skirt[i]);
        glVertex3fv(head[i][0]);
    }
    glVertex3fv(skirt[0]);
    glVertex3fv(head[0][0]);
    
    glEnd();
    glEndList();
    
    return 1;
}


/* main body color 3 */
void draw_ghost (int sprite, int color) {
    glPushMatrix();
    glTranslatef(0.0,0.0,0.5*RADIUS);
    COLORGL(3);
    glCallList(LIST(HEAD));
    if (sprite & 0x01) {
      glRotatef(360.0/16.0,0.0,0.0,1.0);
    }
    glCallList(LIST(SKIRT));
    glPopMatrix();
    return;
}


/* like a ghost but the main body is in color 2 */
/* also has a zigzag mouth (not implemented */
void draw_scared_ghost (int sprite, int color) {
    glPushMatrix();
    glTranslatef(0.0,0.0,0.5*RADIUS);
    COLORGL(2);
    glCallList(LIST(HEAD));
    if (sprite & 0x01) {
      glRotatef(360.0/16.0,0.0,0.0,1.0);
    }
    glCallList(LIST(SKIRT));
    glPopMatrix();
    return;
}
